# -*- coding: utf-8 -*-
require 'open-uri'
require 'nokogiri'
require 'uri'
require 'idealattice/lattice'
require 'idealattice/wiki'

module IdeaLattice
  class Crawler
    def initialize(uri)
      @host, @path, @query, top = paternize(uri)
      @lattice = Lattice.new(top)
      @thread = Thread.start do
        while @lattice.next?
          if $DEBUG
            $stdout.puts("next page=#{@lattice.next_page}")
            $stdout.flush()
          end
          @lattice.add crawle(@lattice.next_page)
          sleep(20)
        end  
      end
    end
    
    def run()
      @thread.join
      File.open("#{@host}.data", "wb") do |f|
        Marshal.dump(@lattice, f)
      end
      puts("wrote #{@lattice.size} pages")
    end
   
    def paternize(uri)
      o = URI.parse(uri)
      if o.query =~ /\A([^;]+);name=(.+)\Z/   # RWiki
        @engine = IdeaLattice::RWiki.new
        return o.host, o.path, "#{$1};name=", $2
      elsif o.path =~ %r|/wiki.cgi|
        @engine = IdeaLattice::YukiWiki.new
        return o.host, o.path, "wiki.cgi?", o.query
      elsif o.path =~ %r|/wiliki.cgi|
        @engine = IdeaLattice::WiLiki.new
        return o.host, o.path, "", o.query
      elsif o.query =~ /\A([0-9A-Za-z]+)\Z/    # WikiWiki, Hiki
        @engine = IdeaLattice::Hiki.new
        return o.host, o.path, "", $1
      else
        raise ArgumentError("#{uri} not supported")
      end
    end
    
    private
    
    def crawle(name)
      url = "http://#{@host}#{@path}?#{@query}#{name}"
      p url if $DEBUG
      begin
        doc = Nokogiri::HTML(open(url))
      rescue
        p $!
        return []
      end  
      pages = []
      doc.search('a').each do |link|
        href = link['href']
        if href =~ /\Ahttp:\/\//
          begin
            u = URI.parse(href)
            next if u.host != @host || u.path != @path
          rescue URI::InvalidURIError
            next
          end  
        end
        @engine.wikiname(href, pages)
        p href, pages if $DEBUG
      end
      pages.uniq
    end
  end
end

if $0 == __FILE__
  c = IdeaLattice::Crawler.new("http://www.artonx.org/collabo/backyard/?FrontPage")
  c.run
end
